/*
 * Authors: David Trowbridge <trowbrds@cs.colorado.edu>
 *
 * Copyright (C) 2004 Novell, Inc (www.novell.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtkbox.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkvbox.h>

#include <libedataserver/e-source.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>

#include "cal-properties.h"
#include "../e-cal-config.h"

struct _prop_data {
	void *object;
	char *name;
	ECalConfig *config;
};

static void
eccp_dialog_response (GtkWidget *dialog, int response, struct _prop_data *prop_data)
{
	if (response == GTK_RESPONSE_OK)
		e_config_commit ((EConfig *) prop_data->config);
	else
		e_config_abort ((EConfig *) prop_data->config);
	gtk_widget_destroy (dialog);
}

static void
colorpicker_set_color (GnomeColorPicker *color, guint32 rgb)
{
	gnome_color_picker_set_i8 (color, (rgb & 0xff0000) >> 16, (rgb & 0xff00) >> 8, rgb & 0xff, 0xff);
}

static guint32
colorpicker_get_color (GnomeColorPicker *color)
{
	guint8 r, g, b, a;
	guint32 rgb = 0;

	gnome_color_picker_get_i8 (color, &r, &g, &b, &a);

	rgb = r;
	rgb <<= 8;
	rgb |= g;
	rgb <<= 8;
	rgb |= b;

	return rgb;
}

static void
copy_properties (const gchar *property, const gchar *value, ESource *source)
{
	g_print ("copying property '%s'\n", property);
	e_source_set_property (source, property, value);
}

static void
eccp_commit (EConfig *ec, GSList *items, void *data)
{
	ECalConfigTargetSource *t = (ECalConfigTargetSource *) ec->target;
	ESource *copy = t->source;
	ESource *source = t->real;
	guint32 color;

	e_source_foreach_property (copy, (GHFunc) copy_properties, source);
	e_source_set_name (source, e_source_peek_name (copy));
	e_source_set_relative_uri (source, e_source_peek_relative_uri (copy));
	e_source_set_absolute_uri (source, e_source_peek_absolute_uri (copy));
	e_source_get_color (copy, &color);
	e_source_set_color (source, color);
}

static void
eccp_free (EConfig *ec, GSList *items, void *data)
{
	struct _prop_data *prop_data = data;

	g_slist_free (items);

	g_free (prop_data);
}

static GtkWidget *
eccp_get_source_type (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *type;
	int row;
	ECalConfigTargetSource *t = (ECalConfigTargetSource *) ec->target;
	ESource *real = t->real;
	ESourceGroup *group = e_source_peek_group (real);
	char *markup;

	if (old)
		return old;

	row = ((GtkTable *)parent)->nrows;

	label = gtk_label_new (_("Type:"));
	gtk_widget_show (label);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	type = gtk_label_new ("");
	gtk_widget_show (type);
	markup = g_strdup_printf ("<span weight=\"bold\">%s</span>", e_source_group_peek_name (group));
	gtk_label_set_markup (GTK_LABEL (type), markup);
	gtk_misc_set_alignment (GTK_MISC (type), 0.0, 0.5);
	g_free (markup);
	gtk_table_attach (GTK_TABLE (parent), type, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	/* HACK */
	gtk_table_set_row_spacings (GTK_TABLE (parent), 6);
	gtk_table_set_col_spacings (GTK_TABLE (parent), 6);

	return type;
}

static void
name_changed (GtkEntry *entry, ECalConfigTargetSource *t)
{
	ESource *source = t->source;
	e_source_set_name (source, gtk_entry_get_text (GTK_ENTRY (entry)));
}

static GtkWidget *
eccp_get_source_name (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *entry;
	int row;
	ECalConfigTargetSource *t = (ECalConfigTargetSource *) ec->target;
	ESource *source = t->source;
	struct _prop_data *prop_data = (struct _prop_data *) data;

	if (old)
		return old;

	row = ((GtkTable*)parent)->nrows;

	label = gtk_label_new_with_mnemonic (_("_Name:"));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	entry = gtk_entry_new ();
	gtk_widget_show (entry);
	gtk_table_attach (GTK_TABLE (parent), entry, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
	g_signal_connect (G_OBJECT (entry), "changed", G_CALLBACK (name_changed), (gpointer) t);

	if (source)
		gtk_entry_set_text (GTK_ENTRY (entry), e_source_peek_name (source));

	return entry;
}

static void
color_changed (GnomeColorPicker *picker, guint r, guint g, guint b, guint a, ECalConfigTargetSource *t)
{
	ESource *source = t->source;
	e_source_set_color (source, colorpicker_get_color (picker));
}

static GtkWidget *
eccp_get_source_color (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *picker;
	int row;
	struct _prop_data *prop_data = data;
	ECalConfigTargetSource *t = ec->target;
	ESource *source = t->source;

	static guint32 assigned_colors[] = {
		0xBECEDD, /* 190 206 221     Blue */
		0xE2F0EF, /* 226 240 239     Light Blue */
		0xC6E2B7, /* 198 226 183     Green */
		0xE2F0D3, /* 226 240 211     Light Green */
		0xE2D4B7, /* 226 212 183     Khaki */
		0xEAEAC1, /* 234 234 193     Light Khaki */
		0xF0B8B7, /* 240 184 183     Pink */
		0xFED4D3, /* 254 212 211     Light Pink */
		0xE2C6E1, /* 226 198 225     Purple */
		0xF0E2EF  /* 240 226 239     Light Purple */
	};
	GRand *rand = g_rand_new ();
	guint32 color;

	if (old)
		return old;

	row = ((GtkTable*)parent)->nrows;

	color = assigned_colors[g_rand_int_range (rand, 0, 9)];
	g_rand_free (rand);

	label = gtk_label_new_with_mnemonic (_("C_olor:"));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	picker = gnome_color_picker_new ();
	gtk_widget_show (picker);
	gtk_label_set_mnemonic_widget (GTK_LABEL (label), picker);
	gtk_table_attach (GTK_TABLE (parent), picker, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect (G_OBJECT (picker), "color-set", G_CALLBACK (color_changed), t);

	if (source)
		e_source_get_color (source, &color);
	else
		/* FIXME */;

	colorpicker_set_color (GNOME_COLOR_PICKER (picker), color);

	return picker;
}

struct _check_data
{
	GtkWidget *dialog;
	EConfig *ec;
};

static void
free_check (struct _check_data *data, GObject *object)
{
	g_object_unref (data->ec);
	g_free (data);
}

static void
check_dialog (ESource *source, struct _check_data *data)
{
	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog),
		GTK_RESPONSE_OK,
		e_config_page_check (data->ec, NULL));
}

static ECalConfigItem eccp_items[] = {
	{ E_CONFIG_BOOK, "", NULL },
	{ E_CONFIG_PAGE,          "00.general", N_("General") },
	{ E_CONFIG_SECTION_TABLE, "00.general/00.source", NULL },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/00.type", NULL, eccp_get_source_type },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/10.name", NULL, eccp_get_source_name },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/20.color", NULL, eccp_get_source_color },
};

/**
 * e_calendar_properties_show:
 * @parent: parent window for dialog (current unused)
 * @source: the ESource corresponding to the calendar
 *
 * Show calendar properties for @source.
 **/
void
e_calendar_properties_show (struct _GtkWindow *parent, ESource *source)
{
	GtkWidget *dialog, *w;
	struct _prop_data *prop_data;
	ECalConfig *ec;
	ECalConfigTargetSource *target;
	ESource *copy;
	GSList *l;
	int i;
	struct _check_data *check;

	if (source == NULL)
		return;

	prop_data = g_malloc0 (sizeof (*prop_data));
	prop_data->object = source;

	dialog = gtk_dialog_new_with_buttons (_("Calendar Properties"), NULL,
	                                      GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_MODAL,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OK, GTK_RESPONSE_OK,
					      NULL);
	gtk_widget_ensure_style (dialog);


	ec = e_cal_config_new (E_CONFIG_BOOK, "org.gnome.evolution.calendar.calendarProperties");
	prop_data->config = ec;
	l = NULL;
	for (i = 0; i < sizeof (eccp_items) / sizeof (eccp_items[0]); i++)
		l = g_slist_prepend (l, &eccp_items[i]);
	e_config_add_items ((EConfig *) ec, l, eccp_commit, NULL, eccp_free, prop_data);

	copy = e_source_copy (source);
	target = e_cal_config_target_new_source (ec, copy, source);
	e_config_set_target ((EConfig *) ec, (EConfigTarget *) target);
	w = e_config_create_widget ((EConfig *) ec);
	gtk_widget_show (w);

	gtk_box_pack_start ((GtkBox *) ((GtkDialog *) dialog)->vbox, w, TRUE, TRUE, 0);

	/* we want to re-check the contents whenever the source changes, so that
	 * we can set the "ok" button sensitivity based on whether things verify */
	check = g_new0 (struct _check_data, 1);
	check->dialog = dialog;
	check->ec = ec;
	g_signal_connect (G_OBJECT (copy), "changed", G_CALLBACK (check_dialog), check);
	g_object_weak_ref (G_OBJECT (dialog), free_check, check);

	/* we do 'apply on ok' ... since instant apply may apply some very long running tasks */
	g_signal_connect (dialog, "response", G_CALLBACK (eccp_dialog_response), prop_data);
	gtk_widget_show (dialog);
}
